==> elisa-plugins-bad_0.5.9-1.orig.tar.gz: elisa-plugins-bad-0.5.9/elisa/plugins/poblesec/history.py <==
# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006-2008 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.

from elisa.core.utils import defer
from elisa.core.log import Loggable
from elisa.core import common

from twisted.python import failure
import gobject

class History(gobject.GObject, Loggable):

    # FIXME: signal name should contain a dash, not an underscore
    __gsignals__ = {'push_controller': 
                              (gobject.SIGNAL_RUN_LAST,
                              gobject.TYPE_BOOLEAN,
                              (gobject.TYPE_PYOBJECT,
                               gobject.TYPE_PYOBJECT,)),
                    'pop_controller': 
                              (gobject.SIGNAL_RUN_LAST,
                              gobject.TYPE_BOOLEAN,
                              (gobject.TYPE_PYOBJECT,
                               gobject.TYPE_PYOBJECT,))
                              }

    def __init__(self, frontend):
        super(History, self).__init__()

        self.frontend = frontend
        self.current = None
        self.index = -1
        self._controllers = []

    def append_controller(self, path, display_name, **kwargs):
        def append_controller(controller):
            controller.display_name = display_name
            self._controllers.append(controller)
            previous = self.current
            self.current = controller
            self.index = self._controllers.index(self.current)
            
            return controller
        
        def emit_push_controller(controller):
            index = self._controllers.index(self.current)
            if index > 0:
                previous = self._controllers[index-1]
            else:
                previous = None
            self.emit('push_controller', previous, self.current)
            
            return controller

        def log_controller_creation_error(error, controller_path):
            log_path = common.application.log_failure(error)
            self.warning("Entering controller with path %s failed. Full" \
                         " failure log at %s" % (controller_path, log_path))
            return error
        
        try:
            dfr = self.frontend.create_controller(path, **kwargs)
        except:
            error = failure.Failure()
            dfr = defer.fail(error)
        
        dfr.addCallback(append_controller)
        dfr.addCallback(emit_push_controller)
        dfr.addErrback(log_controller_creation_error, path)

        return dfr

    def go_back(self):
        if not self.current:
            self.warning("No current controller in this history.")
            return

        # remove the current controller from the history
        try:
            index = self._controllers.index(self.current)
        except ValueError:
            self.warning("Current controller is unknown to this history.")
            return
        self._controllers.pop(index)

        # update the current index
        self.index = index-1

        if index > 0:
            poped_controller = self.current
            previous = self.current
            self.current = self._controllers[index-1]
            self.emit('pop_controller', previous, self.current)

            # call clean method of the controller before losing last reference
            # to it
            previous.clean()

    def clear(self):
        while self.index > 0:
            self.go_back()

==> elisa-plugins-bad_0.5.15.5.orig.tar.gz: elisa-plugins-bad-0.5.15.5/elisa/plugins/poblesec/history.py <==
# -*- coding: utf-8 -*-
# Elisa - Home multimedia server
# Copyright (C) 2006-2008 Fluendo Embedded S.L. (www.fluendo.com).
# All rights reserved.
#
# This file is available under one of two license agreements.
#
# This file is licensed under the GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special
# exception to use Elisa with Fluendo's plugins.
#
# The GPL part of Elisa is also available under a commercial licensing
# agreement from Fluendo.
# See "LICENSE.Elisa" in the root directory of this distribution package
# for details on that license.

from elisa.core.utils import defer
from elisa.core.log import Loggable
from elisa.core import common

import gobject

class History(gobject.GObject, Loggable):

    # FIXME: signal name should contain a dash, not an underscore
    __gsignals__ = {'push_controller': 
                              (gobject.SIGNAL_RUN_LAST,
                              gobject.TYPE_BOOLEAN,
                              (gobject.TYPE_PYOBJECT,
                               gobject.TYPE_PYOBJECT,)),
                    'pop_controller': 
                              (gobject.SIGNAL_RUN_LAST,
                              gobject.TYPE_BOOLEAN,
                              (gobject.TYPE_PYOBJECT,
                               gobject.TYPE_PYOBJECT,))
                              }

    def __init__(self, frontend):
        super(History, self).__init__()

        self._pending_append_deferred = None
        self.frontend = frontend
        self.current = None
        self.index = -1
        self._controllers = []

    def append_controller(self, path, display_name, **kwargs):
        if self._pending_append_deferred != None:
            self._pending_append_deferred.cancel()
            self._pending_append_deferred = None

        def append_controller(controller):
            self._pending_append_deferred = None
            controller.display_name = display_name
            self._controllers.append(controller)
            previous = self.current
            self.current = controller
            self.index = self._controllers.index(self.current)
            
            return controller

        def chaining_failed(failure):
            # the chaining failed because the deferred which is wrapped around
            # the other one has been cancelled
            failure.trap(defer.AlreadyCalledError)
        
        def log_controller_creation_error(error, controller_path):
            self._pending_append_deferred = None
            if error.type != defer.CancelledError:
                # as it was not a CancelledError, we want to log it
                log_path = common.application.log_failure(error)
                self.warning("Entering controller with path %s failed. Full" \
                         " failure log at %s" % (controller_path, log_path))
                # eat CancelledErrors
                return error
       
        try:
            dfr = self.frontend.create_controller(path, **kwargs)

            # chain it together with the cancellable deferred and take of the
            # case that the chaining fails
            cancellable_dfr = defer.Deferred()
            dfr.chainDeferred(cancellable_dfr)
            dfr.addErrback(chaining_failed)

            # store it, so that we can cancel it later
            self._pending_append_deferred = cancellable_dfr

        except:
            cancellable_dfr = defer.fail()

        try:
            previous = self._controllers[-1]
        except IndexError:
            previous = None


        # first the append callback
        cancellable_dfr.addCallback(append_controller)
        
        # then callbacks from somewhere else
        self.emit('push_controller', previous, cancellable_dfr)

        # then errors
        cancellable_dfr.addErrback(log_controller_creation_error, path)

        return cancellable_dfr

    def go_back(self):
        if not self.current:
            self.warning("No current controller in this history.")
            return

        # if there was an append ongoing cancel it
        if self._pending_append_deferred is not None:
            self._pending_append_deferred.cancel()
            self._pendind_append_deferred = None

        # remove the current controller from the history
        try:
            index = self._controllers.index(self.current)
        except ValueError:
            self.warning("Current controller is unknown to this history.")
            return
        self._controllers.pop(index)

        # update the current index
        self.index = index-1

        if index > 0:
            poped_controller = self.current
            previous = self.current
            self.current = self._controllers[index-1]
            self.emit('pop_controller', previous, self.current)

            # call clean method of the controller before losing last reference
            # to it
            previous.clean()

    def clear(self):
        while self.index > 0:
            self.go_back()
